--[[
	EasyShards v0.05 by EVmaker

	A simple mod for helping with the management of excess soulshards.
	Also allows the hiding of the Soulbag when opening all bags.
]]--

-- Declare globals
ES_Settings = {
["Version"] = "0.05",
["OnOff"] = true,
["UseMMB"] = true,
["Warn"] = false,
["HideSoulBag"] = false,
["RightClick"] = "deleteshard",
["CountColor"] = {16,255,246},
["ShardLimit"] = 0
}

-- Declare locals
local ES_EML = "EasyShards";
local debugMode = false;
--local playerClass = false;
local ES_defaultSettings = {
["Version"] = "0.05",
["OnOff"] = true,
["UseMMB"] = true,
["Warn"] = false,
["HideSoulBag"] = false,
["RightClick"] = "deleteshard",
["CountColor"] = {16,255,246},
["ShardLimit"] = 0
}

-- Slash handling
function ES_SlashHandler(theMsg)
	msg = strtrim(strlower(theMsg or ""))
	if msg == "version" then
		EMLChat("This version of EasyShards is: "..ES_defaultSettings["Version"],"chat",ES_EML)
	elseif strmatch(msg, "rightclick") then
		local theChoice = strmatch(msg, "rightclick (.*)");
		if (theChoice == "clearshards") then
			ES_Settings["RightClick"] = "clearshards";
			EMLChat("Right clicking the minimap button/ldb will now clear extra soulshards.","chat",ES_EML)
		elseif (theChoice == "deleteshard") then
			ES_Settings["RightClick"] = "deleteshard";
			EMLChat("Right clicking the minimap button/ldb will now delete a soulshard.","chat",ES_EML)
		else
			EMLChat("Current options for right-clicking the minimap button/ldb are: clearshards, and deleteshard","chat",ES_EML)
			EMLChat("clearshards: Clears non-soulbag soulshards, or ones above the limit if no soulbag.","chat",ES_EML)
			EMLChat("deleteshard: deletes a single soulshard.","chat",ES_EML)
		end
	elseif msg == "menu" then
		EMLFrame_Toggle("ES1")
	elseif strmatch(msg, "toggle") then
		local theChoice = strmatch(msg, "toggle (.*)");
		if (theChoice ~= nil and theChoice ~= "" and theChoice ~= " ") then
			ES_Toggle(theChoice)
		else
			EMLChat("Availible toggles are: status, onoff, warn, minimap","chat",ES_EML)
		end
	elseif msg == "reset" then
		ES_Reset()
	elseif strmatch(msg, "shardlimit") then
		local theChoice = tonumber(strmatch(msg, "shardlimit (.*)"));
		local isClear = strmatch(msg, "shardlimit (.*)");
		if (theChoice) and (isClear ~= "clear") then
			ES_Settings["ShardLimit"]=theChoice;
			EMLChat("The Shard Limit is now "..tostring(theChoice),"chat",ES_EML)
		elseif (isClear == "clear") then
			ES_Settings["ShardLimit"]=0;
			EMLChat("Cleared the Shard Limit.","chat",ES_EML)
		else
			EMLChat("Must enter a number for the Shard Limit","chat",ES_EML)			
		end
		ES_setCheckBoxes()
	elseif msg == "debug" then
		if (debugMode) then
			debugMode = false;
			EMLChat("Toggling Debug Mode off","chat",ES_EML)
		else
			EMLChat("Toggling Debug Mode on","chat",ES_EML)
			debugMode = true;
		end
	elseif msg == "count" then
		local shardCount,_,_ = ES_shardInfo();
		EMLChat("You have "..tostring(shardCount).." soul shards in your bags.","chat",ES_EML)
	elseif strmatch(msg, "test1") then
		local theChoice = strmatch(msg, "test1 (.*)");
		ES_testFunction1(theChoice)
	elseif strmatch(msg, "test2") then
		local theChoice = strmatch(msg, "test2 (.*)");
		ES_testFunction2(theChoice)
	else
		EMLChat("toggle, (toggles options on or off)","chat",ES_EML)
		EMLChat("version, (check your EasyShards version)","chat",ES_EML)
		EMLChat("menu, (opens the GUI configuration menu)","chat",ES_EML)
		EMLChat("count, (tells how many shards are in your bags)","chat",ES_EML)
		EMLChat("rightclick, (lets you choose what happens on right clicking the minimap button)","chat",ES_EML)
	end
end

function EasyShards_OnLoad()
	local playerClass = tostring(UnitClass("player"));
	if (playerClass == "Warlock") then OpenAllBags = ES_openAllBags; end
	this:RegisterEvent("PLAYER_ENTERING_WORLD")
	this:RegisterEvent("ITEM_PUSH")
	this:RegisterEvent("BAG_UPDATE")
	SLASH_ES1 = "/easyshards"
	SLASH_ES2 = "/es"
	SlashCmdList["ES"] = function(msg)
		ES_SlashHandler(msg);
	end
	EMLChat("EasyShards "..ES_defaultSettings["Version"].." loaded","plainchat",ES_EML,"teal")
end

function EasyShards_OnEvent(event)
	if (event == "PLAYER_ENTERING_WORLD") then
		-- Setting stuff up
		local theR,theG,theB = EML_splitTable(ES_Settings["CountColor"]);
		ESSC = ES_MinimapButton:CreateFontString("ESSC","ARTWORK","GameFontNormal")
		ESSC:SetAllPoints()
		ESSC:SetTextColor(theR,theG,theB,1)
		ES_setCheckBoxes()
		if (ES_Settings["Version"] ~= ES_defaultSettings["Version"]) then
			ES_Reset("version")
		end
	end
	if (event == "ITEM_PUSH" and ES_Settings["OnOff"] and ES_Settings["Warn"]) then
		local shardLimit = tonumber(ES_Settings["ShardLimit"]);
		local soulShardCount,hasSoulBag,soulSize = ES_shardInfo();
		if (soulSize) or (not hasSoulBag and shardLimit ~= 0 and soulShardCount > shardLimit) then
			if (soulSize) then
				EMLChat("You have "..(soulShardCount-soulSize).." excess soulshards.","chat",ES_EML,"red")
			else
				EMLChat("You have "..(soulShardCount-shardLimit).." excess soulshards.","chat",ES_EML,"red")
			end
		end
	end
	if (event == "BAG_UPDATE" and ES_Settings["OnOff"] and (not ES_Settings["Warn"])) then
		ES_clearShards()
		ES_buttonUpdate()
	end
end

-- Resets settings
function ES_Reset(arg)
	local localSettings = EML_copyTable(ES_defaultSettings);
	ES_Settings = EML_mergeTable(ES_Settings,localSettings);
	if (arg == "version") then
		ES_Settings["Version"]=localSettings["Version"];
		EMLChat("Version is changed, updating settings.","chat",ES_EML)
	else
		EMLChat("Resetting settings","chat",ES_EML)
		EML_clearTable(ES_Settings)
		ES_Settings=localSettings;
	end
	ES_setCheckBoxes()
end

-- Checks settings
function ES_checkCheckBoxes()
	local userInput=tonumber(ES1_ShardLimit:GetText());
	if (userInput ~= nil and userInput ~= "" and userInput ~= " ") then
		ES_Settings["ShardLimit"]=tonumber(userInput);
	else
		ES_Settings["ShardLimit"]=0;
	end
	if (ES1_OnOff:GetChecked()) then
		ES_Settings["OnOff"]=true;
	else
		ES_Settings["OnOff"]=false;
	end
	if (ES1_HideMMB:GetChecked()) then
		ES_Settings["UseMMB"]=true;
		ES_MinimapButton:Show()
	else
		ES_Settings["UseMMB"]=false;
		ES_MinimapButton:Hide()
	end
	if (ES1_Warn:GetChecked()) then
		ES_Settings["Warn"]=false;
	else
		ES_Settings["Warn"]=true;
	end
	if (ES1_HideSoulBag:GetChecked()) then
		ES_Settings["HideSoulBag"]=true;
	else
		ES_Settings["HideSoulBag"]=false;
	end
	ES_buttonUpdate()
end

-- Sets settings
function ES_setCheckBoxes()
	if (ES_Settings["ShardLimit"] ~= 0) then
		ES1_ShardLimit:SetText(ES_Settings["ShardLimit"])
	else
		ES1_ShardLimit:SetText(0)
	end
	if (ES_Settings["OnOff"]) then
		ES1_OnOff:SetChecked(true)
	else
		ES1_OnOff:SetChecked(nil)
	end
	if (ES_Settings["UseMMB"]) then
		ES1_HideMMB:SetChecked(true)
		ES_MinimapButton:Show()
	else
		ES1_HideMMB:SetChecked(nil)
		ES_MinimapButton:Hide()
	end
	if (ES_Settings["Warn"]) then
		ES1_Warn:SetChecked(nil)
	else
		ES1_Warn:SetChecked(true)
	end
	if (ES_Settings["HideSoulBag"]) then
		ES1_HideSoulBag:SetChecked(true)
	else
		ES1_HideSoulBag:SetChecked(nil)
	end
	ES_buttonUpdate()
end

-- Updates the minimap button counter
function ES_buttonUpdate()
	local theR,theG,theB = EML_splitTable(ES_Settings["CountColor"]);
	ESSC:SetTextColor(theR,theG,theB,1)
	ESSC:SetText(GetItemCount(6265))
end

-- Determines what right-clicking the minimap button does
function ES_rightClick()
	if (ES_Settings["RightClick"] == "clearshards") then
		ES_clearShards()
	elseif (ES_Settings["RightClick"] == "deleteshard") then
		ES_deleteSingleShard()
	else
		if (debugMode) then EMLChat("No right-click functionality defined.","chat",ES_EML,"red") end
	end
end

-- Gets shard information
function ES_shardInfo()
	local soulShardCount = GetItemCount(6265)
	local _,bagType,bagSlots,_ = EML_bagInfo();
	local hasSoulBag = false;
	local soulSize = nil;
	for i=0, #(bagSlots) do
		if (bagType[i] == 4) then hasSoulBag = i; end
	end
	if (hasSoulBag) then
		soulSize = EML_getTable(hasSoulBag,bagSlots);
	end
	return soulShardCount,hasSoulBag,soulSize;
end

-- Clears excess shards
function ES_clearShards()
	local shardLimit = tonumber(ES_Settings["ShardLimit"]);
	local _,bagType,bagSlots = EML_bagInfo();
	local soulShardCount,hasSoulBag,soulSize = ES_shardInfo();
	if (soulSize) or (not hasSoulBag and shardLimit ~= 0 and soulShardCount > shardLimit) then
		for k=0, #(bagType) do
			if (k ~= hasSoulBag) then
				for i=0, bagSlots[k] do
					if (debugMode) then EMLChat("Count: "..tostring(soulShardCount)..", Bag: "..tostring(k)..", Slot: "..tostring(i),"chat",ES_EML) end
					if (not soulSize and shardLimit ~= 0) then
						if (soulShardCount > shardLimit) then
							local didDelete = ES_deleteShard(k,i);
							if (didDelete) then soulShardCount = soulShardCount - 1; end
						end
					end
					if (soulSize and shardLimit ~= 0) then
						if (soulShardCount > (soulSize+shardLimit)) then
							local didDelete = ES_deleteShard(k,i);
							if (didDelete) then soulShardCount = soulShardCount - 1; end
						end
					elseif (soulSize and shardLimit == 0) then
						ES_deleteShard(k,i)
					end
				end
			end
		end
	else
		if (debugMode) then EMLChat("Not clearing","chat",ES_EML) end
	end
end

-- Deletes a single shard
function ES_deleteSingleShard()
	local _,_,bagSlots = EML_bagInfo();
	local _,hasSoulBag,_ = ES_shardInfo();
	local didDelete = false;
	for k=0, #(bagSlots) do
		if (k ~= hasSoulBag) then
			for i=0, bagSlots[k] do
				if (debugMode) then EMLChat("Count: "..tostring(soulShardCount)..", Bag: "..tostring(k)..", Slot: "..tostring(i),"chat",ES_EML) end
				local didDelete = ES_deleteShard(k,i);
				if (didDelete) then
					return true;
				end
			end
		end
	end
	return false;
end

-- Deletes the Shard
function ES_deleteShard(bagID,slotID)
	local _,itemNum = EML_itemInfo(bagID,slotID);
	if (itemNum == 6265) then
		if (debugMode) then EMLChat("Deleting Shard at: Bag "..tostring(bagID)..", Slot "..tostring(slotID)..".","chat",ES_EML) end
		PickupContainerItem(bagID,slotID)
		DeleteCursorItem()
		return true;
	end
end

function ES_testFunction1(arg)
	EMLChat("Test Function 1:","chat",ES_EML,"teal")
	EML_Display(ES_Settings,ES_EML)
end

function ES_testFunction2(arg)
	EMLChat("Test Function 2:","chat",ES_EML,"teal")
	local totalBags = NUM_BAG_SLOTS+NUM_BANKBAGSLOTS;
	EML_Display(totalBags,ES_EML)
	for i=1,totalBags do
		OpenBag(i)
	end
end

function ES_openAllBags(allOpen)
	local allBagsOpen = true;
	local _,soulBag,_ = ES_shardInfo();
	local totalBags = NUM_BAG_SLOTS+NUM_BANKBAGSLOTS;
	for i=0,totalBags do
		if (i ~= soulBag) then
			if (not IsBagOpen(i)) then
				allBagsOpen = false;
			end
		end
		CloseBag(i)
	end
	if (allOpen or not allBagsOpen) then
		if (ES_Settings["HideSoulBag"]) then
			ES_openBags("hidesoul")
		else
			ES_openBags()
		end
	end
end

function ES_openBags(arg)
	local _,soulBag,_ = ES_shardInfo();
	local totalBags = NUM_BAG_SLOTS+NUM_BANKBAGSLOTS;
	for i=0,totalBags do
		if (i ~= soulBag and arg == "hidesoul") then
			OpenBag(i)
		elseif (arg ~= "hidesoul") then
			OpenBag(i)
		end
	end
end

function ES_Toggle(arg)
	if (arg == "onoff") then
		if (ES_Settings["OnOff"]) then
			ES_Settings["OnOff"] = false;
			EMLChat("Toggling EasyShards off","chat",ES_EML)
		else
			ES_Settings["OnOff"] = true;
			EMLChat("Toggling EasyShards on","chat",ES_EML)
		end
		ES_setCheckBoxes()
	elseif (arg == "minimap") then
		if (ES_Settings["UseMMB"]) then
			ES_Settings["UseMMB"] = false;
			EMLChat("Toggling the minimap button off","chat",ES_EML)
		else
			ES_Settings["UseMMB"] = true;
			EMLChat("Toggling the minimap button on","chat",ES_EML)
		end
		ES_setCheckBoxes()
	elseif (arg == "warn") then
		if (ES_Settings["Warn"]) then
			ES_Settings["Warn"] = false;
			EMLChat("Will now automatically delete excess shards","chat",ES_EML)
		else
			ES_Settings["Warn"] = true;
			EMLChat("Will now warn when there are excess shards instead of deleting them","chat",ES_EML)
		end
		ES_setCheckBoxes()
	elseif (arg == "hidesoulbag") then
		if (ES_Settings["HideSoulBag"]) then
			ES_Settings["HideSoulBag"] = false;
			EMLChat("Will open the soulbag when opening all bags","chat",ES_EML)
		else
			ES_Settings["HideSoulBag"] = true;
			EMLChat("Will not open the soulbag when opening all bags","chat",ES_EML)
		end
		ES_setCheckBoxes()
	elseif (arg == "status") then
		if (ES_Settings["OnOff"]) then
			EMLChat("EasyShards is on.","chat",ES_EML)
		else
			EMLChat("EasyShards is off.","chat",ES_EML)
		end
		if (ES_Settings["UseMMB"]) then
			EMLChat("Minimap button is shown.","chat",ES_EML)
		else
			EMLChat("Minimap button is hidden.","chat",ES_EML)
		end
		if (ES_Settings["Warn"]) then
			EMLChat("Warning when there are excess shards instead of deleting.","chat",ES_EML)
		else
			EMLChat("Deleting excess shards.","chat",ES_EML)
		end
		if (ES_Settings["HideSoulBag"]) then
			EMLChat("Hiding the Soulbag when opening all bags.","chat",ES_EML)
		else
			EMLChat("Soulbag will open when opening all bags.","chat",ES_EML)
		end
	else
		EMLChat("Availible toggles are: status, onoff, warn, minimap","chat",ES_EML)
	end
end

function ES_GUIPickColor()
	EML_colorPicker("open")
end

function ES_GUISetColor()
	local pickedColor = EML_colorPicker();
	ES_Settings["CountColor"] = pickedColor;
	EMLChat("Count Color set to this color.","chat",ES_EML,pickedColor)
	ES_buttonUpdate();
end
-- End of code